import { Between, EntityRepository, FindOperator, getConnection, Like, Repository } from "typeorm";
import CryptoUtils from "../../utils/crypto.utils";
import { CreateUserDto } from "./dto/create-user.dto";
import { UpdateUserDto } from "./dto/update-user.dto";
import { User } from "./entities/user.entity";
import { removeSpecialCharacters } from "src/helper/common-helper";

@EntityRepository(User)
export class UsersRepository extends Repository<User> {
    async createEntity(dto: CreateUserDto): Promise<User | Error> {
        try {
            const { firstName, lastName, birthdayDate, email, phone, password: pass } = dto;

            let userType = "";
            let role = 3;
            if (dto.userType && dto.userType === "admin") {
                userType = dto.userType;
                role = 2;
            } else userType = "customer";

            const entity = this.create({
                firstName: removeSpecialCharacters(firstName),
                lastName: removeSpecialCharacters(lastName),
                birthdayDate,
                email,
                phone,
                password: await CryptoUtils.encrypt(pass),
                role,
                userType
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateUserDto): Promise<User | Error> {
        try {
            const user = await this.findOne(id);
            await this.update(id, {
                firstName: dto.firstName ? dto.firstName : user.firstName,
                lastName: dto.lastName ? dto.lastName : user.lastName,
                birthdayDate: dto.birthdayDate ? dto.birthdayDate : user.birthdayDate,
                phone: dto.phone ? dto.phone : user.phone,
                image: dto.image ? dto.image : user.image
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAllEntities(): Promise<User[] | Error> {
        try {
            return await this.find();
        } catch (e) {
            return e;
        }
    }

    async findByQuery(query: any): Promise<User[] | Error> {
        try {
            query = { ...query, isDeleted: false };
            return await this.find({
                order: { createdAt: -1 },
                where: query
            });
        } catch (e) {
            return e;
        }
    }

    async findUserForDropDown(searchKeyword?: any): Promise<User[] | Error> {
        try {
            type FindOption = {
                name?: FindOperator<string>;
            };

            const findOptions: FindOption = {};

            if (searchKeyword) findOptions.name = Like("%" + searchKeyword + "%");

            // return await this.find({
            //     select :["id","name"],
            //     where: {...findOptions, isDeleted: false, isApproved: true}
            // });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<User | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findOneUserDetailsEntity(request: any): Promise<User | Error> {
        try {
            return await this.findOne({
                where: {
                    id: request.user.id,
                    role: request.user.role,
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findByEmailAddress(email: string): Promise<User | Error> {
        try {
            return await this.findOne({
                where: { email: email, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async findByEmailAddressAndActive(
      email: string,
      role: number
    ): Promise<User | Error> {
        try {
            return await this.findOne({
                where: { email: email, role: role, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
              .createQueryBuilder()
              .update(User)
              .set({ isDeleted: true })
              .where("id = :id", { id: id })
              .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return false;
        }
    }

    async updateUserPassword(
      id: string,
      password: string
    ): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
              .createQueryBuilder()
              .update(User)
              .set({ password: password })
              .where("id = :id", { id: id })
              .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return false;
        }
    }

    async countCustomer(status: number): Promise<any | Error> {
        try {
            return await this.count({
                where: { isDeleted: false, role: status }
            });
        } catch (e) {
            return e;
        }
    }

    async monthlyCustomer(): Promise<any | Error> {
        try {
            let monthWiseCount: any = [];
            const date = new Date();
            for (let i = 0; i < 12; i++) {
                const fyear = new Date(date.getFullYear(), i, 1).toLocaleString(
                  "default",
                  { year: "numeric" }
                );
                const fmonth = new Date(date.getFullYear(), i, 1).toLocaleString(
                  "default",
                  { month: "2-digit" }
                );
                const fday = new Date(date.getFullYear(), i, 1).toLocaleString(
                  "default",
                  { day: "2-digit" }
                );

                // Generate yyyy-mm-dd date string
                const firstDayFormattedDate = fyear + "-" + fmonth + "-" + fday;

                const lyear = new Date(date.getFullYear(), i + 1, 0).toLocaleString(
                  "default",
                  { year: "numeric" }
                );
                const lmonth = new Date(date.getFullYear(), i + 1, 0).toLocaleString(
                  "default",
                  { month: "2-digit" }
                );
                const lday = new Date(date.getFullYear(), i + 1, 0).toLocaleString(
                  "default",
                  { day: "2-digit" }
                );

                // Generate yyyy-mm-dd date string
                const lastDayFormattedDate = lyear + "-" + lmonth + "-" + lday;

                console.log("First Day: ", firstDayFormattedDate);
                console.log("Last Day: ", lastDayFormattedDate);

                const start = new Date(firstDayFormattedDate);
                start.setUTCHours(0, 0, 0, 0);
                const end = new Date(lastDayFormattedDate);
                end.setUTCHours(23, 59, 59, 999);
                const data = await this.count({
                    where: {
                        isDeleted: false,
                        createdAt: Between(
                          new Date(start).toISOString(),
                          new Date(end).toISOString()
                        )
                    }
                });
                monthWiseCount.push(data ? data : 0);
            }
            return monthWiseCount;
        } catch (e) {
            return e;
        }
    }
}
